/**
* 数字证书
*/
define("project/scripts/account/digitalCertificate",function(require,exports,module){
	var appUtils = require("appUtils"),
		gconfig = require("gconfig"),
		global = gconfig.global,
		layerUtils = require("layerUtils"),
		shellPlugin = require("shellPlugin"),
		publicMethod = require("PublicMethod"),
		utils = require("utils"),
		_pageId = "#account_digitalCertificate",
		downloadProgressInterval = null;
		
	var timeoutInt=10;
	var clearTimeout=null;
	
	function init()
	{
		$(_pageId+" .header .icon_back").hide();
		installCertificate();
		//设置超时
		timeoutFun();
		
	}
	
	function bindPageEvent()
	{
		// 绑定FAQ常见问题按钮
		publicMethod.bindProblemEvent(_pageId);
		//联系电话
		appUtils.bindEvent($(_pageId+" #a_phone"),function(){
			layerUtils.iAlert("开户热线：4000-188-288",'','',"知道了");
		});
	
		appUtils.bindEvent($(_pageId+" .header .icon_back"),function(){
			appUtils.pageBack();
		});
	}
	
	function destroy()
	{
		$(_pageId+" .order_succes h5").html("正在安装证书,请勿退出应用");
		$(_pageId+" .order_succes div").show();
		$(_pageId+" .cert_down").hide();
		$(_pageId+" #next_btn a").html("请稍后...");
		$(_pageId+" #next_btn").attr("class","ce_btn");
		$(_pageId+" #next_btn a").off();
	}
	
	function timeoutFun(){
		if(clearTimeout){
			window.clearTimeout(clearTimeout);
		}
		clearTimeout=window.setTimeout(function(){
			//appUtils.pageInit("account/digitalCertificate","account/accountResult",{});
			publicMethod.openAccountResult("account/digitalCertificate");
		},timeoutInt*1000);
	}
	
	function queryProtocol()
	{
		var queryProtocolParam = {
				"funcNo" : 1001013,
				"user_id" : appUtils.getSStorageInfo("user_id"),
				"clientinfo" : appUtils.getSStorageInfo("clientinfo"),
				"jsessionid" : appUtils.getSStorageInfo("jsessionid")
			};
		appUtils.invokeServer(global.serverPath,queryProtocolParam,function(data){
			if(data.error_no == 0)
			{
				var protocolId = data.results[0].protocol_id,
					protocolMd5 = data.results[0].protocol_md5;
				signProtocols(protocolId,protocolMd5);
			}
			else
			{
				layerUtils.iLoading(false);
				layerUtils.iAlert(data.error_info);
			}
		},false);
	}
	
	function signProtocols(protocolId,protocolMd5)
	{
		layerUtils.iLoading(true);
		// 协议的数组
		var protocolArray = new Array(),
			signParam = {
				"medid" : protocolId,
				"content" : protocolMd5,
				"userId" : appUtils.getSStorageInfo("user_id")
			};
		signParam.userId += "_zd";
		// 获取协议的数字签名值
		shellPlugin.callShellMethod("signPlugin",function(data){
			// 添加值到数组中
			var protocol = {
					"protocol_id" : protocolId,
					// 数字签名值
					"protocol_dcsign" : data.ciphertext,
					"summary" : protocolMd5
			};
			protocolArray.push(protocol);
			var signProtocolParam = {
					"funcNo" : 501554,
					"user_id" : appUtils.getSStorageInfo("user_id"),
					"jsondata" : JSON.stringify(protocolArray),
					"ipaddr" : appUtils.getSStorageInfo("ip"),
					"macaddr" : appUtils.getSStorageInfo("mac"),
					"clientinfo" : appUtils.getSStorageInfo("clientinfo"),
					"jsessionid" : appUtils.getSStorageInfo("jsessionid")
			};
			appUtils.invokeServer(global.serverPath,signProtocolParam,function(data){
				if(data.error_no == 0)
				{
					modifyUserSource();
				}
				else
				{
					layerUtils.iLoading(false);
					layerUtils.iAlert(data.error_info);
				}
			},false);
		},null,signParam);
	}
	
	function modifyUserSource()
	{
		// 修改用户来源信息
		var modifyUserSourceParam = {
				"funcNo" : 1001005,
				"userid" : appUtils.getSStorageInfo("user_id"),
				"clientinfo" : appUtils.getSStorageInfo("clientinfo"),
				"jsessionid" : appUtils.getSStorageInfo("jsessionid")
		};
		appUtils.invokeServer(global.serverPath,modifyUserSourceParam,function(data){
			if(data.error_no == 0)
			{
				//appUtils.pageInit("account/digitalCertificate","account/accountResult",{});
				publicMethod.openAccountResult("account/digitalCertificate");
			}
			else
			{
				layerUtils.iAlert(data.error_info);
			}
		});
	}
	
	function installCertificate()
	{
		// 十位随机数
		var createKeyRandom = "1";
		for(var i=0;i<9;i++)
		{
			createKeyRandom += parseInt(Math.random()*10)+"";
		}
		// 证书申请串，数据来源于壳子
		var pkcs10 = "",
			createKeyParam = {
				"rodam" : createKeyRandom,
				"userId" : appUtils.getSStorageInfo("user_id"),
				// 通讯唯一标识
				"key" : "pingan"
			};
		createKeyParam.userId += "_zd";
		// 调用壳子的方法生成证书申请串
		shellPlugin.callShellMethod("createKeyPlugin",function(data){
			pkcs10 = data.pkcs10;
			// 中登证书
			var param = {
					"funcNo" : 501529,
					"user_id" : appUtils.getSStorageInfo("user_id"),
					"pkcs10" : encodeURIComponent(pkcs10),
					"clientinfo" : appUtils.getSStorageInfo("clientinfo"),
					"jsessionid" : appUtils.getSStorageInfo("jsessionid")
			};
			// 监听超时
			utils.listenTimeout(dealInstallFail,false);
			appUtils.invokeServer(global.serverPath,param,function(data){
				utils.listenTimeout(null,true);
				if(data.error_no == "0")
				{
					var certParam = {
							// 获取证书的内容
							"content" : data.results[0].p7cert,
							"userId" : appUtils.getSStorageInfo("user_id")
						};
					certParam.userId += "_zd";
					// 壳子读取证书，然后安装证书
					shellPlugin.callShellMethod("initZsPlugin",function(data){
						if(data == "OK")
						{
							$(_pageId+" .order_succes h5").html("数字证书安装成功");
							$(_pageId+" .order_succes div").hide();
							$(_pageId+" .cert_down").show();
							$(_pageId+" .cert_stat").html("安装成功");
//							$(_pageId+" #next_btn a").html("下一步");
							$(_pageId+" #next_btn").hide();
							$(_pageId+" #next_btn").attr("class","ct_btn");
							queryProtocol();
						}
					},dealInstallFail,certParam);
				}
				else
				{
					dealInstallFail();
					layerUtils.iAlert(data.error_info);
				}
			},true,true,false);
		},null,createKeyParam);
	}
	
	function dealInstallFail()
	{
		$(_pageId+" #next_btn").attr("class","ct_btn");
		// 隐藏照片
		$(_pageId+" .order_succes div").hide();
		$(_pageId+" #next_btn a").html("重 试");
		$(_pageId+" .order_succes h5").html("数字证书安装失败");
		$(_pageId+" .cert_down").show();
		$(_pageId+" .cert_stat").html("安装失败");
		appUtils.bindEvent($(_pageId+" #next_btn a"),function(){
			destroy();
			init();
		});
	}
	
	var digitalCertificate = {
		"init" : init,
		"bindPageEvent" : bindPageEvent,
		"destroy" : destroy
	};
	
	module.exports = digitalCertificate;
});