/**
 * 通用工具类，将项目中的公用方法抽出来
 */
define(function(require,exports,module){
	
	var appUtils = require("appUtils"),
		layerUtils = require("layerUtils"),
		shellPlugin = require("shellPlugin"),
		validatorUtil = require("validatorUtil"),
		gconfig = require("gconfig"),
		layer = require("layer"),
		global = gconfig.global;
	
	/**
	 * @param isLastReq 是否是最后一次请求
	 * @param isAsync 是否是异步的
	 */
	function installCertificate(isLastReq,isAsync)
	{
		// 十位随机数
		var createKeyRandom = "1";
		for(var i=0;i<9;i++)
		{
			createKeyRandom += parseInt(Math.random()*10)+"";
		}
		// 证书申请串，数据来源于壳子
		var pkcs10 = "",
			createKeyParam = {
				"rodam" : createKeyRandom,
				"userId" : appUtils.getSStorageInfo("user_id"),
				// 通讯唯一标识
				"key" : "pingan"
			};
		// 调用壳子的方法生成证书申请串
		shellPlugin.callShellMethod("createKeyPlugin",function(data){
			pkcs10 = data.pkcs10;
			// 中登证书
			var param = {
					"funcNo" : 501529,
					"user_id" : appUtils.getSStorageInfo("user_id"),
					"pkcs10" : encodeURIComponent(pkcs10),
					"clientinfo" : appUtils.getSStorageInfo("clientinfo"),
					"jsessionid" : appUtils.getSStorageInfo("jsessionid")
			};
			appUtils.invokeServer(global.serverPath,param,function(data){
				if(data.error_no == "0")
				{
					var certParam = {
							// 获取证书的内容
							"content" : data.results[0].p7cert,
							"userId" : appUtils.getSStorageInfo("user_id")
						};
					// 壳子读取证书，然后安装证书
					shellPlugin.callShellMethod("initZsPlugin",function(data){
						// 关闭等待层。。。
						layerUtils.iLoading(false);
					},null,certParam);
				}
				else
				{
					layerUtils.iLoading(false);
					layerUtils.iAlert(data.error_info);
				}
			},isLastReq,isAsync);
		},null,createKeyParam);
	}
	
	/**
	 * 自定义确认框
	 * @param layerMsg 确认框的内容
	 * @param oneText 第一个按钮按钮的文本内容
	 * @param twoText 第二个按钮的文本内容
	 * @param funcOne 第一个按钮的回调函数
	 * @param funcTwo 第二个按钮的回调函数
	 */
	function layerTwoButton(layerMsg,oneText,twoText,funcOne,funcTwo){
		var viewContent = '<div class="pop_tip notice" id="utils_confirm"><span class="icon"></span><p>'+layerMsg+'</p><div class="btn"><a href="javascript:void(0);" id="utils_confirm_one">'+oneText+'</a><a href="javascript:void(0);" id="utils_confirm_two">'+twoText+'</a></div></div>';
		var iConfirm = layerUtils.layerCustom(viewContent);
		appUtils.preBindEvent($("#utils_confirm")," #utils_confirm_one",function(){
			if(funcOne)
			{
				funcOne();
			}
			layerUtils.iCustomClose();
		});
		appUtils.preBindEvent($("#utils_confirm")," #utils_confirm_two",function(){
			if(funcTwo)
			{
				funcTwo();
			}
			layerUtils.iCustomClose();
		});
	}
	
	/**
	 * 超时处理
	 * @param func 超时处理函数
	 * @param isSuccess 请求是否成功
	 * gconfig.ajaxTimeout 是配置文件中配置的超时时间
	 */
	var catchTimeout = null;
	function listenTimeout(func,isSuccess)
	{
		if(!isSuccess)
		{
			if(catchTimeout != null)
			{
				window.clearTimeout(catchTimeout);
				catchTimeout = null;
			}
			catchTimeout = window.setTimeout(function(){
				// 关闭加载层
				layerUtils.iLoading(false);
				func();
			},(gconfig.ajaxTimeout+5)*1000);
		}
		else
		{
			window.clearTimeout(catchTimeout);
			catchTimeout = null;
		}
	}
	
	/**
	 * 通过壳子调用系统日期控件
	 * @param obj dom 对象，一般传 this 即可
	 */
	function selectDate(obj)
	{
		var currentDateStr = $(obj).val(),
			yearParam = "",
			monthParam = "",
			dayParam = "";
		currentDateStr = currentDateStr.replace(/-/g,"/");
		if(validatorUtil.isDate(currentDateStr))    // 如果是有效的日期串，解析年、月、日
		{
			var tempDate = new Date(currentDateStr);
			yearParam = tempDate.getFullYear()+"";
			// 0~11 的值
			monthParam = (tempDate.getMonth()+1+"").length == 1 ? "0"+(tempDate.getMonth()+1) : (tempDate.getMonth()+1)+"";
			// 1~31 的值
			dayParam = (tempDate.getDate()+"").length == 1 ? "0"+tempDate.getDate() : tempDate.getDate()+"";
		}
		var calendarParam = {
				"selector" : $(obj).attr("id"),
				"year" : yearParam,
				"month" : monthParam,
				"day" : dayParam
		};
		shellPlugin.callShellMethod("calendarPlugin", function (data) {
			// 回调数据中的 data 是字符串，需解析
			if (typeof(data) == "string") {
				data = JSON.parse(data);
			}
			$("#" + data.selector).val(data.date);
		}, null, calendarParam);
	}
	
	/**
	 * 以 放大镜 的形式显示数字，如：电话号码、银行卡号
	 * @param obj 要处理的 dom 对象，一般传 this
	 * @param type 1 手机号码，2 银行卡
	 */
	function showBigNo(obj,type)
	{
		var showStr = $(obj).val();
		var originalStr = showStr.replace(/\s+/g,"");
		var styleStr = "background-color:#FFFFFF; color:#000000; border:1px solid #D83005;line-height:35px;" +
			"font-size:20px;padding:0px 5px;font-weight:bold;";
		if(type == 1)
		{
			if(originalStr.length > 3 && originalStr.length <= 7)
			{
				showStr = originalStr.substring(0,3)+" "+originalStr.substring(3,7);
			}
			else if(originalStr.length > 7 && originalStr.length <= 11)
			{
				showStr = originalStr.substring(0,3)+" "+originalStr.substring(3,7)+" "+originalStr.substring(7,11);
			}
			// 浏览器
			if(gconfig.platform == 0)
			{
				styleStr += "width:150px;";
			}
			// android
			else if(gconfig.platform == 1)
			{
				styleStr += "width:140px;";
			}
			// ios
			else 
			{
				styleStr += "width:146px;";
			}
		}
		else if(type == 2)
		{
			if(originalStr.length > 4 && originalStr.length <= 8)
			{
				showStr = originalStr.substring(0,4)+" "+originalStr.substring(4,8);
			}
			else if(originalStr.length > 8 && originalStr.length <= 12)
			{
				showStr = originalStr.substring(0,4)+" "+originalStr.substring(4,8)+" "+originalStr.substring(8,12);
			}
			else if(originalStr.length > 12 && originalStr.length <= 16)
			{
				showStr = originalStr.substring(0,4)+" "+originalStr.substring(4,8)+" "+originalStr.substring(8,12)+
					" "+originalStr.substring(12,16);
			}
			else if(originalStr.length > 16 && originalStr.length <= 19)
			{
				showStr = originalStr.substring(0,4)+" "+originalStr.substring(4,8)+" "+originalStr.substring(8,12)+
					" "+originalStr.substring(12,16)+" "+originalStr.substring(16);
			}
			
			if(originalStr.length <= 16)
			{
				// 浏览器
				if(gconfig.platform == 0)
				{
					styleStr += "width:212px;";
				}
				// 安卓手机
				else if(gconfig.platform == 1)
				{
					styleStr += "width:188px;";
				}
				// ios
				else 
				{
					styleStr += "width:198px;";
				}
			}
			else
			{
				// 浏览器
				if(gconfig.platform == 0)
				{
					styleStr += "width:253px;";
				}
				// 安卓手机
				else if(gconfig.platform == 1)
				{
					styleStr += "width:225px;";
				}
				// ios
				else 
				{
					styleStr += "width:235px;";
				}
			}
		}
//		$(obj).val(showStr);
		var tipsIndex = $.layer({
				type : 4,
				closeBtn : false,
				shade : false,
				shadeClose : false,
				fix : true,
				title : false,
				area : ['auto','auto'],
				border : [10 , 0.3 , '#000', true],
				zIndex : 19930902,
				tips : {
					msg: showStr,
					follow: obj,
					guide: 0,
					isGuide: true,
					style: [styleStr, '#D83005']
				}
		});
		var widthDifference = ($(".xubox_tips").offset().left + $(".xubox_tips").width()) -  $(window).width();
		if(widthDifference > 0)
		{
			$(".xubox_tips").css("margin-left","-"+(widthDifference+10)+"px");
			$(".xubox_tips").find("i").css("margin-left",(widthDifference+10)+"px");
		}
		if(originalStr == "" && tipsIndex)
		{
			layer.close(tipsIndex);
		}
		appUtils.bindEvent($(obj),function(){
			if(tipsIndex)
			{
				layer.close(tipsIndex);
			}
		},"blur");
	}
	
	/**
	 * 判断是否是身份证号
	 * @param num 身份证号
	 */
	function isIdCardNo(num)
	{  
		num = (num+"").toUpperCase(); 
		//身份证号码为15位或者18位，15位时全为数字，18位前17位为数字，最后一位是校验位，可能为数字或字符X。  
		if (!(/(^\d{15}$)|(^\d{17}([0-9]|X)$)/.test(num)))  
		{
			return '输入的身份证号长度不对！';
		}
		//校验位按照ISO 7064:1983.MOD 11-2的规定生成，X可以认为是数字10。
		//下面分别分析出生日期和校验位
		var len, re;
		len = num.length;
		if (len == 15)
		{
			re = new RegExp(/^(\d{6})(\d{2})(\d{2})(\d{2})(\d{3})$/);
			var arrSplit = num.match(re);

			//检查生日日期是否正确
			var dtmBirth = new Date('19' + arrSplit[2] + '/' + arrSplit[3] + '/' + arrSplit[4]+" 10:00:00");
			var bGoodDay;
			bGoodDay = (dtmBirth.getYear() == Number(arrSplit[2])) && ((dtmBirth.getMonth() + 1) == Number(arrSplit[3])) && (dtmBirth.getDate() == Number(arrSplit[4]));
				if (!bGoodDay)
				{
				  return '输入的身份证号格式不对！';
				}
				else
				{
				//将15位身份证转成18位
				//校验位按照ISO 7064:1983.MOD 11-2的规定生成，X可以认为是数字10。
				  var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
				  var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
				  var nTemp = 0, i;  
					num = num.substr(0, 6) + '19' + num.substr(6, num.length - 6);
				   for(i = 0; i < 17; i ++)
				  {
						nTemp += num.substr(i, 1) * arrInt[i];
				   }
				   num += arrCh[nTemp % 11];  
					return true;  
				}  
		}

		if (len == 18)
		{
			re = new RegExp(/^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})([0-9]|X)$/);
			var arrSplit = num.match(re);

			//检查生日日期是否正确
			var dtmBirth = new Date(arrSplit[2] + "/" + arrSplit[3] + "/" + arrSplit[4]+" 10:00:00");
			var bGoodDay;
			bGoodDay = (dtmBirth.getFullYear() == Number(arrSplit[2])) && ((dtmBirth.getMonth() + 1) == Number(arrSplit[3])) && (dtmBirth.getDate() == Number(arrSplit[4]));
			if (!bGoodDay)
			{
				return '输入的身份证号格式不对！';
			}
			else
			{
				//检验18位身份证的校验码是否正确。
				//校验位按照ISO 7064:1983.MOD 11-2的规定生成，X可以认为是数字10。
				var valnum;
				var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2);
				var arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2');
				var nTemp = 0, i;
				for(i = 0; i < 17; i ++)
				{
					nTemp += num.substr(i, 1) * arrInt[i];
				}
				valnum = arrCh[nTemp % 11];
				if (valnum != num.substr(17, 1))
				{
					return '输入的身份证号格式不对！';
				}
				return true;  
			}
		}
		return false;
	}
	
	/**
	 * 处理 iPhone 的兼容性
	 * @param obj 待处理的 input dom 对象，一般传 this
	 * @param maxLength 最大长度
	 */
	function dealIPhoneMaxLength(obj,maxLength)
	{
		var currentStr = $(obj).val();
		if(currentStr.length > maxLength)
		{
			$(obj).val(currentStr.substring(0,maxLength));
		}
	}
	
	/**
	 * 设置输入框光标的位置
	 * obj 需要设置光标位置的 input dom 对象
	 * position 光标的位置，默认是输入框的末尾
	 */
	function setCursorPosition(obj,position)
	{
	   var text = obj.createTextRange(); 
	   text.collapse(true); 
	   text.moveStart("character",position ? position : obj.value.length); 
	   text.select();
	}
	
	/**
	 * 将 key1=value1&key2=value2 转换为 json 对象，返回值为转换后的 json 对象
	 * @param data 需要解析的字符串
	 */
	function getJSONObject(data)
	{
		var object = {},
			dataArray = data.split("&"),
			oneData = null;
		for(var i = 0;i < dataArray.length; i++)
		{
			oneData = dataArray[i].split("=");
			object[onedata[0]] = onedata[1];
		}
		return object;
	}
	
	/**
	 * 调用原生打电话的功能
	 * phone手机号  
	 */
	function callKHRXPhone(phone){
		var param = {
				"tel" : phone
			};
		shellPlugin.callShellMethod("callPhonePlugin", null,null,param);
	}
	
	var utils = {
		"installCertificate" : installCertificate,
		"layerTwoButton" : layerTwoButton,
		"selectDate" : selectDate,
		"showBigNo" : showBigNo,
		"dealIPhoneMaxLength" : dealIPhoneMaxLength,
		"setCursorPosition" : setCursorPosition,
		"isIdCardNo" : isIdCardNo,
		"listenTimeout" : listenTimeout,
		"getJSONObject" : getJSONObject,
		"callKHRXPhone" : callKHRXPhone
	};
	
	module.exports = utils;
});